﻿/******************************************************************************
 *  
 *  Filename:    BdlcListCreator.cs
 *  Description: This class creates a BDLC enabled List and stores its settings
 *               information in the current BdlcConfiguration List
 *  Author:      Jörn Kornberger
 *  Date:        28.10.2010
 *             
 *   Copyright Layer 2 GmbH 2010. All rights reserved.     
 *     
 ******************************************************************************/
using System;
using Layer2.SharePoint.UI.Administration.Lists;
using Microsoft.SharePoint;

namespace Layer2.ExternalBdlcApi
{
    public class BdlcListCreator
    {
        /// <summary>
        /// This method will create a List in the given web and will enable it as BDLC List.
        /// The list gets new columns as needed from the given Select statement.
        /// It will be enabled for background update and writeback if needed.
        /// A new configuration entry will be created in the BdlcConfiguration List.
        /// </summary>
        /// <remarks>
        /// IMPORTANT: Only use on SiteCollections with activated BDLC feature!
        /// </remarks>
        /// <param name="web">The web in which the new list will be created</param>
        /// <param name="listName">The name of the list</param>
        /// <param name="enableWriteBack">flag if writeback has to be enabled</param>
        /// <param name="enableBackgroundUpdate">flag if the background update has to be enabled</param>
        /// <param name="connectionstring">the connectionstring to the datasource</param>
        /// <param name="connectionProvider">the connectionprovider as string (e.g. "System.Data.SqlClient" )</param>
        /// <param name="selectStatement">The SQL select statement</param>
        /// <param name="primaryKey">primary key</param>
        public void AddBdlcEnabledList(SPWeb web, string listName, bool enableWriteBack, bool enableBackgroundUpdate, string connectionstring,
            string connectionProvider, string selectStatement, string primaryKey)
        {

            Guid listId = web.Lists.Add(listName, String.Empty, SPListTemplateType.GenericList);
            SPList list = web.Lists[listId];
            list.OnQuickLaunch = true;
            list.Update();

            ListSettings listSettings = new ListSettings(
                connectionstring,
                connectionProvider,
                selectStatement,
                primaryKey,
                web.ID,
                listId,
                enableBackgroundUpdate,
                enableWriteBack,
                false);

            var businessDataListConnectorLogic = new BusinessDataListConnectorLogic();
            businessDataListConnectorLogic.Init(web, list, listSettings);
            businessDataListConnectorLogic.ExpandList();
            businessDataListConnectorLogic.SaveSettings();
            businessDataListConnectorLogic.EnableOrDisableWriteBack(listSettings.Sync);
        }
    }
}
