﻿using System;
using System.Collections.Generic;
using Layer2.SharePoint.UI.Administration.Lists;
using Layer2.SharePoint.UI.Administration.Lists.Security;
using Microsoft.SharePoint;

namespace Layer2.Products.BdlcApiExamples
{
    class Maintenance
    {
        /// <summary>
        /// When using the BDLC API from within a web service, the write back throws errors and the update was errored out. 
        /// The API now provides a method to register your webservice to BDLC. Best is to register the asmx address of your service.
        /// </summary>
        /// <param name="site"></param>
        /// <param name="listId"></param>
        /// <param name="webId"></param>
        internal static void RegisterWebServiceAndUpdateListInBdlc(SPSite site, Guid listId, Guid webId)
        {
            BusinessDataListConnectorLogic logic = new BusinessDataListConnectorLogic();
            SPWeb web = site.OpenWeb(webId);
            SPList list = web.Lists[listId];
            logic.Init(web, list);
            logic.RegisterCustomUrl("BdlcService.asmx"); // <= this is an example value, insert your webservice here!
            string result = logic.UpdateList();
        }

        /// <summary>
        /// This method will update the BDLC config list item with the given new properties for provider, connection and query. 
        /// </summary>
        /// <param name="bdlcConfigList">the BDLC Configuration list. 
        /// Best read the GUID of it before and access it through _web.Lists.GetList(configListId, true)</param>
        /// <param name="listToUpdate">The BDLC connected list in question</param>
        /// <param name="provider">the new provider value</param>
        /// <param name="connectionString">the new connection string in plain text</param>
        /// <param name="selectStatement">the new query string in plain text</param>
        internal void UpdateBdlcListConnectionItem(SPList bdlcConfigList, SPList listToUpdate,
            string provider, string connectionString, string selectStatement)
        {
            string queryString =
                "<Where><And><Eq><FieldRef Name='{0}' /><Value Type='Text'>{1}</Value></Eq><Eq><FieldRef Name='{2}' /><Value Type='Text'>{3}</Value></Eq></And></Where><OrderBy><FieldRef Name='ID' /></OrderBy>";
            queryString = String.Format(queryString, "bdlcListId", listToUpdate.ID, "bdlcWebId", listToUpdate.ParentWeb.ID);
            SPQuery query = new SPQuery { Query = queryString };

            var results = bdlcConfigList.GetItems(query);
            if (results == null || results.Count == 0)
                return;

            string encryptedConnectionString = RijndaelCiphering.Encrypt(connectionString);
            string encryptedselectStatement = RijndaelCiphering.Encrypt(selectStatement);

            SPListItem item = results[0];
            item[Constants.SELECT_STATEMENT] = encryptedselectStatement;
            item[Constants.CONNECTION_PROVIDER] = provider;
            item[Constants.CONNECTION_STRING] = encryptedConnectionString;
            item.Update();
        }

        /// <summary>
        /// Checks all BDLC connections for invalid or corrupted locks and releases them if necessary
        /// </summary>
        /// <param name="site">your site collection</param>
        /// <returns></returns>
        internal static string MaintainConnectionLocks(SPSite site) //other possible parameter is the rootweb 
        {
            var businessDataListConnectorLogic = new BusinessDataListConnectorLogic();
            return businessDataListConnectorLogic.EnsureGeneralUnlock(site);
        }

        internal static void RepairWriteback(SPSite site) 
        {
            BusinessDataListConnectorLogic.RepairWriteBackOnAllLists(site);
        }

        internal void AllApiMethods(SPList list, SPWeb web, SPSite site)
        {
            // variables used as parameters
            string url = string.Empty;
            List<string> listOfUrls = new List<string>();
            DateTime startTime = DateTime.Now;
           
            // BDLC API variable
            ListSettings listSettings = new ListSettings();

            var logic = new BusinessDataListConnectorLogic();
            
            logic.Init(list);
            logic.Init(web, list);
            logic.Init(web, list, listSettings);

            // the following methods need the Init() methods first
            string result = logic.InitialWriteback();
            logic.EnsureUnlock(); 
            bool exists = logic.ConfigurationListExists();
            exists = logic.ConfigurationListExists(web);
            logic.EnableOrDisableListEvents(true);
            logic.EnableOrDisableWorkflowActivation(true);
            logic.EnableOrDisableWriteBack(true);         
            logic.ExpandList();
            listSettings = logic.LoadSettings();
            logic.SaveSettings();
            logic.SaveSettings(listSettings, true);  // deprecated
           
            result = logic.UpdateList();
            result = logic.UpdateList(listSettings);
            result = logic.UpdateListNotSilenced();
          
            // use without Init
            logic.UpdateAllLists(web);
            logic.EnsureWriteBackOnAllLists(site);
            result = logic.EnsureGeneralUnlock(web);
            result = logic.EnsureGeneralUnlock(site);
            logic.RegisterCustomUrl(url);
            logic.RegisterCustomUrl(listOfUrls);

            // static methods
            BusinessDataListConnectorLogic.RepairWriteBackOnAllLists(site);

            // usage of encryption to add encrypted values to Bdlc config list.
            string encryptedValue = RijndaelCiphering.Encrypt("HelloWorld");

        }
    }
}
